@@ -1,5 +1,6 @@
 import logging
 from collections import OrderedDict
+from collections.abc import Iterator
 import formulas  # type: ignore
 
 from typing import (
@@ -14,22 +15,23 @@ from typing import (
 from business_rule_engine.exceptions import (
     DuplicateRuleName,
     MissingArgumentError,
-    ConditionReturnValueError
+    ConditionReturnValueError,
+    RuleParserSyntaxError,
 )
 
 
 class Rule():
 
-    def __init__(self, rulename, condition_requires_bool: bool = True) -> None:
+    def __init__(self, rulename: Text, condition_requires_bool: bool = True) -> None:
         self.condition_requires_bool = condition_requires_bool
         self.rulename: Text = rulename
         self.conditions: List[Text] = []
         self.actions: List[Text] = []
-        self.status = None
+        self.status: Optional[bool] = None
 
     @staticmethod
     def _compile_condition(condition_lines: List[Text]) -> Any:
-        condition = "".join(condition_lines)
+        condition = " ".join(condition_lines)
         if not condition.startswith("="):
             condition = "={}".format(condition)
         return formulas.Parser().ast(condition)[1].compile()  # type: ignore
@@ -52,7 +54,7 @@ class Rule():
         params_condition = {k: v for k, v in params_dict.items() if k in condition_args}
         return params_condition
 
-    def check_condition(self, params, *, set_default_arg=False, default_arg=None):
+    def check_condition(self, params: Dict[Text, Any], *, set_default_arg: bool = False, default_arg: Any = None) -> Any:
         condition_compiled = self._compile_condition(self.conditions)
         params_condition = self._get_params(params, condition_compiled, set_default_arg, default_arg)
         rvalue_condition = condition_compiled(**params_condition).tolist()
@@ -61,12 +63,12 @@ class Rule():
         self.status = bool(rvalue_condition)
         return rvalue_condition
 
-    def run_action(self, params, *, set_default_arg=False, default_arg=None):
+    def run_action(self, params: Dict[Text, Any], *, set_default_arg: bool = False, default_arg: Any = None) -> Any:
         action_compiled = self._compile_condition(self.actions)
         params_actions = self._get_params(params, action_compiled, set_default_arg, default_arg)
         return action_compiled(**params_actions)
 
-    def execute(self, params, *, set_default_arg=False, default_arg=None) -> Tuple[bool, Any]:
+    def execute(self, params: Dict[Text, Any], *, set_default_arg: bool = False, default_arg: Any = None) -> Tuple[Any, Any]:
         rvalue_condition = self.check_condition(params, set_default_arg=set_default_arg, default_arg=default_arg)
         if not self.status:
             return rvalue_condition, None
@@ -83,10 +85,11 @@ class RuleParser():
         self.condition_requires_bool = condition_requires_bool
 
     def parsestr(self, text: Text) -> None:
-        rulename = None
-        is_condition = False
-        is_action = False
-        ignore_line = False
+        rulename: Optional[Text] = None
+        is_condition: bool = False
+        is_action: bool = False
+        is_then: bool = False
+        ignore_line: bool = False
 
         for line in text.split('\n'):
             ignore_line = False
@@ -103,6 +106,9 @@ class RuleParser():
                 is_condition = True
                 is_action = False
             if line.lower().strip().startswith('then'):
+                if is_then:
+                    raise RuleParserSyntaxError('using multiple "then" in one rule is not allowed')
+                is_then = True
                 ignore_line = True
                 is_condition = False
                 is_action = True
@@ -110,17 +116,27 @@ class RuleParser():
                 ignore_line = True
                 is_condition = False
                 is_action = False
+                is_then = False
             if rulename and is_condition and not ignore_line:
                 self.rules[rulename].conditions.append(line.strip())
             if rulename and is_action and not ignore_line:
                 self.rules[rulename].actions.append(line.strip())
 
+    def add_rule(self, rulename: Text, condition: Text, action: Text) -> None:
+        if rulename in self.rules:
+            raise DuplicateRuleName("Rule '{}' already exists!".format(rulename))
+        rule = Rule(rulename)
+        rule.conditions.append(condition)
+        rule.actions.append(action)
+        self.rules[rulename] = rule
+
     @classmethod
     def register_function(cls, function: Any, function_name: Optional[Text] = None) -> None:
-        cls.CUSTOM_FUNCTIONS.append(function_name or function.__name__.upper())
-        formulas.get_functions()[function_name or function.__name__.upper()] = function  # type: ignore
+        custom_function_name = function_name or function.__name__
+        cls.CUSTOM_FUNCTIONS.append(custom_function_name.upper())
+        formulas.get_functions()[custom_function_name.upper()] = function  # type: ignore
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator:
         return self.rules.values().__iter__()
 
     def execute(
